(* Copyright (C) 2014
   Patryk Czarnik  <czarnik@mimuw.edu.pl>
   Jacek Chrząszcz <chrzaszcz@mimuw.edu.pl>
   Aleksy Schubert <alx@mimuw.edu.pl>
*)

(** @author Patryk Czarnik *)
(** Module enumerates arithmetic operators supported by the semantics. *)
Module ArithmeticOperators.

  (** Unary arithmetic operators. *)
  Inductive TUnOp :=
  | UnOp_neg.

  (** Binary arithmetic operators. *)
  Inductive TBinOp :=
  | BinOp_add
  | BinOp_sub
  | BinOp_mul
  | BinOp_div
  | BinOp_rem
  | BinOp_xor
  | BinOp_and
  | BinOp_or
  | BinOp_shl
  | BinOp_shr
  | BinOp_ushr.

  (** Comparison arithmetic operations. *)
  Inductive TCmpAOp :=
  | FCmpOp_fcmpg
  | FCmpOp_fcmpl.

  (** Comparison operators. *)
  Inductive TCmpOp :=
  | CmpOp_eq
  | CmpOp_ne
  | CmpOp_le
  | CmpOp_lt
  | CmpOp_ge
  | CmpOp_gt.

  (** Casting operators. *)
  Inductive TCastOp :=
  | CastOp_b
  | CastOp_c
  | CastOp_s
  | CastOp_i
  | CastOp_l
  | CastOp_f
  | CastOp_d.

  (** Comparison operators returning integer (for fcmpl and fcmpg
      as well as lcmp).
   *)
  Inductive TCmpiOp :=
  | CmpiOp_cmpl
  | CmpiOp_cmpg.
End ArithmeticOperators.
