(* Copyright (C) 2014
   Patryk Czarnik  <czarnik@mimuw.edu.pl>
   Jacek Chrząszcz <chrzaszcz@mimuw.edu.pl>
   Aleksy Schubert <alx@mimuw.edu.pl>
*)
Add LoadPath "IEEE754" as IEEE754.
Require Export IEEE754.IEEE.
Require Import IEEE754.Diadic.
Require Export ZArith.
Open Local Scope Z_scope.

(** @author Aleksy Schubert *)

(* TODO: find the information on how the conversions work in JVMS *)
(* Read about extended-exponent floating-point value set http://docs.oracle.com/javase/specs/jvms/se7/html/jvms-2.html#jvms-2.3.2 *)

 (** Conversion operations for IEEE 754 *)
Module Type FCONVERT.

  (** Abstract type of floating point numbers of various precisions *)
  Parameter t : Set.
  Parameter toIEEE : t -> abstract_IEEE.

  (** Converts a number (third parameter) of the given floating number
      type (first parameter) to one in the resulting format (third
      parameter). *)
  Parameter float_to_float : float_type -> float_type -> t -> t.

  (** Converts a number (third parameter) of the given floating number
  type (first parameter) to one in the resulting integer format (third
  parameter). *)

  Parameter float_to_int : float_type -> Z -> t -> Z.


  (** Converts a number (third parameter) of the given integer 
  type (first parameter) to one in the resulting floating point format (third
  parameter). *)
  Parameter int_to_float : Z -> float_type -> Z -> t.

End FCONVERT.



Module FConv <: FCONVERT.

  Definition t := abstract_IEEE.
  Definition toIEEE : t -> abstract_IEEE := fun x => x.


  (** Converts a number (third parameter) of the given floating number
      type (first parameter) to one in the resulting format (third
      parameter). *)
  Parameter float_to_float : float_type -> float_type -> t -> t.

  (** Converts a number (third parameter) of the given floating number
  type (first parameter) to one in the resulting integer format (third
  parameter). *)

  Parameter float_to_int : float_type -> Z -> t -> Z.


  (** Converts a number (third parameter) of the given integer 
  type (first parameter) to one in the resulting floating point format (third
  parameter). *)
  Parameter int_to_float : Z -> float_type -> Z -> t.

End FConv.
