(* Copyright (C) 2014
   Patryk Czarnik  <czarnik@mimuw.edu.pl>
   Jacek Chrząszcz <chrzaszcz@mimuw.edu.pl>
   Aleksy Schubert <alx@mimuw.edu.pl>
*)

(** * Program example

  @author Patryk Czarnik

  An example program represented in CoJaq.
*)

Require Import Common.
Require Import ValuesNTypes.
Require Import ProgramStructures.
Require Import Heap.
Require Import ZArith.
Require Import Program.
Require Import RuntimeStructures.
Require Import Semantics.
Require Import ArithOps.
Require Import List.
Require Import Relations.
Import List.ListNotations.

Declare Module P : PROGRAM_W_CONSTRUCTORS.
Module PS := P.M_ProgramStructures.
Module VAT := PS.M_ValuesAndTypes.
Module INum := VAT.M_Numeric_Int.
Module PSFacts := ProgramStructuresFacts PS.

Import P PS PSFacts VAT.

Parameter n: Z.

(*=== program ===*)

Definition A_m_code: P.TCode := P.codeFromList
  [(* 0*) I_Frame (FI_Stackop (SI_Const KInt (VInt (INum.zero))));
   (* 1*) I_Frame (FI_Var (VI_Store VIKInt var0));
   (* 2*) I_Frame (FI_Stackop (SI_Const KInt (VInt (INum.const 50))));
   (* 3*) I_Frame (FI_Var (VI_Store VIKInt var1));
   (* 4*) I_Frame (FI_Stackop (SI_Const KInt (VInt (INum.zero))));
   (* 5*) I_Frame (FI_Var (VI_Store VIKInt var2));
   (* 6*) I_Frame (FI_Var (VI_Load  VIKInt var0)); 
   (* 7*) I_Frame (FI_Var (VI_Load  VIKInt var1));
   (* 8*) I_Frame (FI_Cond (CI_Cmp KInt ArithmeticOperators.CmpOp_ge (offsetFromPosition 19%nat)));
   (* 9*) I_Frame (FI_Var (VI_Load  VIKInt var0));
   (*10*) I_Frame (FI_Var (VI_Load  VIKInt var2));
   (*11*) I_Frame (FI_Stackop (SI_Binop KInt ArithmeticOperators.BinOp_add));
   (*12*) I_Frame (FI_Var (VI_Store VIKInt var2));
   (*13*) I_Frame (FI_Var (VI_Inc var0 (INum.one)));
   (*14*) I_Frame (FI_Var (VI_Load  VIKInt var0));
   (*15*) I_Frame (FI_Var (VI_Load  VIKInt var2));
   (*16*) I_Frame (FI_Stackop (SI_Binop KInt ArithmeticOperators.BinOp_add));
   (*17*) I_Frame (FI_Var (VI_Store VIKInt var2));
   (*18*) I_Frame (FI_Cond (CI_Goto (offsetFromPosition 6%nat)));
   (*19*) I_Frame (FI_Var (VI_Load  VIKInt var2));
   (*20*) I_Return (Some KInt)
  ].

